
#ifndef MCS_H  /* this keeps this header from getting rolled in more than once */
#define MCS_H

#include <stdio.h>
#include <time.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/un.h>
#include <netinet/in.h> /* for network sockets */
#include <arpa/inet.h>  /* for network sockets */



#define LWA_IP_MSE   "127.0.0.1" /* IP address of MCS Scheduler "ms_exec" process */
#define LWA_PORT_MSE 9734        /* port for MCS Scheduler "ms_exec" process */
#define LWA_PORT_MSE2 9735        /* port for MCS Scheduler "ms_mdre_ip" process */

#define MIB_LABEL_FIELD_LENGTH 32
#define MIB_VAL_FIELD_LENGTH 8192 /* changed from 256 on 100701 for v.0.6.1 */

/* for IP exchanges between MCS/Exec and ms_mdre_ip */
struct LWA_mib_entry {
  char ss[4];                          /* three-character subsystem designator */
  char label[MIB_LABEL_FIELD_LENGTH];  /* MIB label */
  char val[MIB_VAL_FIELD_LENGTH];      /* MIB value, as char. */
  struct timeval last_change;          /* Indicates time this entry was last changed */
                       /* .tv_sec is seconds into current epoch */
                       /* .tv_usec is fractional remainder in microseconds */
  };

/* Using approach of bit-flagging (note power-of-two values) so these can be added: */
#define LWA_MIBERR_OK         0 /* no error to report */
#define LWA_MIBERR_CANTOPEN   1 /* couldn't open MIB dbm */
#define LWA_MIBERR_CANTSTORE  2 /* couldn't store to MIB dbm */
#define LWA_MIBERR_REF_UNK    4 /* REFERENCE was unrecognized, so not sure what this message is */
                                /* in response to.  Other than SUMMARY, MIB may not have been */
                                /* properly updated. */
#define LWA_MIBERR_CANTFETCH  8 /* couldn't fetch from MIB dbm */
#define LWA_MIBERR_SID_UNK   16 /* Subsystem ID (3-char) was unrecognized. Other than SUMMARY, */
                                /* MIB may not have been properly updated. */
#define LWA_MIBERR_SID_CID   32 /* Command is something this subsystem shouldn't have supported */
                                /* e.g., NU# doing something other than PNG, RPT, or SHT */
                                /* e.g., Receiving MCS as a subsystem ID */  
                                /* e.g., Subsystem MIB handler (ms_mcic_XXX.c) didn't recognize command */ 
#define LWA_MIBERR_OTHER     64 /* MIB may be out of sync for other reasons.  For example: */
                                /* - PTQ timeout, so not sure if command was acted up or not */ 

/* Subsystem SUMMARY (MIB 1.1) values */
#define LWA_SIDSUM_NULL    0 
#define LWA_SIDSUM_NORMAL  1
#define LWA_SIDSUM_WARNING 2
#define LWA_SIDSUM_ERROR   3
#define LWA_SIDSUM_BOOTING 4
#define LWA_SIDSUM_SHUTDWN 5

/* === LWA Subsystem IDs === */
#define LWA_MAX_SID 18 /* maximum subsystem code; also maximum number of subsystems */
#define LWA_SID_NU1  1 /* null subsystem #1 (used for testing) */
#define LWA_SID_NU2  2 /* null subsystem #2 (used for testing) */
#define LWA_SID_NU3  3 /* null subsystem #3 (used for testing) */
#define LWA_SID_NU4  4 /* null subsystem #4 (used for testing) */
#define LWA_SID_NU5  5 /* null subsystem #5 (used for testing) */
#define LWA_SID_NU6  6 /* null subsystem #6 (used for testing) */
#define LWA_SID_NU7  7 /* null subsystem #7 (used for testing) */
#define LWA_SID_NU8  8 /* null subsystem #8 (used for testing) */
#define LWA_SID_NU9  9 /* null subsystem #9 (used for testing) */
#define LWA_SID_MCS 10 /* MCS */
#define LWA_SID_SHL 11 /* SHL (shelter) */
#define LWA_SID_ASP 12 /* ASP */
#define LWA_SID_DP_ 13 /* DP */
#define LWA_SID_DR1 14 /* MCS-DR #1 */
#define LWA_SID_DR2 15 /* MCS-DR #2 */
#define LWA_SID_DR3 16 /* MCS-DR #3 */
#define LWA_SID_DR4 17 /* MCS-DR #4 */
#define LWA_SID_DR5 18 /* MCS-DR #5 */
/* When adding subsystems, remember to change LWA_MAX_SID ! */

int LWA_getsid( char *ssc ) {
  /* ssc is the three-character subsystem code */
  /* returns the LWA subsystem ID, or 0 if there is an error */
  int sid = 0;
  if (!strcmp(ssc,"NU1")) sid = LWA_SID_NU1;
  if (!strcmp(ssc,"NU2")) sid = LWA_SID_NU2;
  if (!strcmp(ssc,"NU3")) sid = LWA_SID_NU3;
  if (!strcmp(ssc,"NU4")) sid = LWA_SID_NU4;
  if (!strcmp(ssc,"NU5")) sid = LWA_SID_NU5;
  if (!strcmp(ssc,"NU6")) sid = LWA_SID_NU6;
  if (!strcmp(ssc,"NU7")) sid = LWA_SID_NU7;
  if (!strcmp(ssc,"NU8")) sid = LWA_SID_NU8;
  if (!strcmp(ssc,"NU9")) sid = LWA_SID_NU9;
  if (!strcmp(ssc,"MCS")) sid = LWA_SID_MCS;
  if (!strcmp(ssc,"SHL")) sid = LWA_SID_SHL;
  if (!strcmp(ssc,"ASP")) sid = LWA_SID_ASP;
  if (!strcmp(ssc,"DP_")) sid = LWA_SID_DP_;
  if (!strcmp(ssc,"DR1")) sid = LWA_SID_DR1;
  if (!strcmp(ssc,"DR2")) sid = LWA_SID_DR2;
  if (!strcmp(ssc,"DR3")) sid = LWA_SID_DR3;
  if (!strcmp(ssc,"DR4")) sid = LWA_SID_DR4;
  if (!strcmp(ssc,"DR5")) sid = LWA_SID_DR5;
  return sid;
  } /* LWA_getsid() */
 
/* === LWA Command (TYPE) IDs === */
#define LWA_MAX_CMD  33 /* maximum code; also maximum number of commands */
#define LWA_CMD_MCSSHT 0 /* Not a subsystem command.  Directs ms_mcic to shutdown */
#define LWA_CMD_PNG    1 /* PNG */
#define LWA_CMD_RPT    2 /* RPT */
#define LWA_CMD_SHT    3 /* SHT */
#define LWA_CMD_INI    4 /* INI (SHL,ASP,DP_,DR#) */
#define LWA_CMD_TMP    5 /* TMP (SHL) */
#define LWA_CMD_DIF    6 /* DIF (SHL) */
#define LWA_CMD_PWR    7 /* PWR (SHL) */
#define LWA_CMD_FIL    8 /* FIL (ASP) */
#define LWA_CMD_AT1    9 /* AT1 (ASP) */
#define LWA_CMD_AT2   10 /* AT2 (ASP) */
#define LWA_CMD_ATS   11 /* ATS (ASP) */
#define LWA_CMD_FPW   12 /* FPW (ASP) */
#define LWA_CMD_RXP   13 /* RXP (ASP) */
#define LWA_CMD_FEP   14 /* FEP (ASP) */
#define LWA_CMD_TBW   15 /* TBW (DP_) */
#define LWA_CMD_TBN   16 /* TBN (DP_) */
#define LWA_CMD_DRX   17 /* DRX (DP_) */
#define LWA_CMD_BAM   18 /* BAM (DP_) */
#define LWA_CMD_FST   19 /* FST (DP_) */
#define LWA_CMD_CLK   20 /* CLK (DP_) */
#define LWA_CMD_REC   21 /* REC (DR#) */
#define LWA_CMD_DEL   22 /* DEL (DR#) */
#define LWA_CMD_STP   23 /* STP (DR#) */
#define LWA_CMD_GET   24 /* GET (DR#) */
#define LWA_CMD_CPY   25 /* CPY (DR#) */
#define LWA_CMD_DMP   26 /* DMP (DR#) */
#define LWA_CMD_FMT   27 /* FMT (DR#) */
#define LWA_CMD_DWN   28 /* DWN (DR#) */
#define LWA_CMD_UP_   29 /* UP_ (DR#).  Note ICD calls this "UP" (no underscore) */
#define LWA_CMD_SEL   30 /* SEL (DR#) */
#define LWA_CMD_SYN   31 /* SYN (DR#) */
#define LWA_CMD_TST   32 /* TST (DR#) */
#define LWA_CMD_BUF   33 /* BUF (DR#) */


/* When adding commands, remember to change LWA_MAX_CMD ! */

int LWA_getcmd( char *ssc ) {
  /* ssc is the three-character command (TYP) */
  /* returns the LWA command ("TYPE"), or 0 if there is an error */
  int cmd = 0;
  if (!strcmp(ssc,"PNG")) cmd = LWA_CMD_PNG;
  if (!strcmp(ssc,"RPT")) cmd = LWA_CMD_RPT;
  if (!strcmp(ssc,"SHT")) cmd = LWA_CMD_SHT;
  if (!strcmp(ssc,"INI")) cmd = LWA_CMD_INI;
  if (!strcmp(ssc,"TMP")) cmd = LWA_CMD_TMP;
  if (!strcmp(ssc,"DIF")) cmd = LWA_CMD_DIF;
  if (!strcmp(ssc,"PWR")) cmd = LWA_CMD_PWR;
  if (!strcmp(ssc,"FIL")) cmd = LWA_CMD_FIL;
  if (!strcmp(ssc,"AT1")) cmd = LWA_CMD_AT1;
  if (!strcmp(ssc,"AT2")) cmd = LWA_CMD_AT2;
  if (!strcmp(ssc,"ATS")) cmd = LWA_CMD_ATS;
  if (!strcmp(ssc,"FPW")) cmd = LWA_CMD_FPW;
  if (!strcmp(ssc,"RXP")) cmd = LWA_CMD_RXP;
  if (!strcmp(ssc,"FEP")) cmd = LWA_CMD_FEP;
  if (!strcmp(ssc,"TBW")) cmd = LWA_CMD_TBW;
  if (!strcmp(ssc,"TBN")) cmd = LWA_CMD_TBN;
  if (!strcmp(ssc,"DRX")) cmd = LWA_CMD_DRX;
  if (!strcmp(ssc,"BAM")) cmd = LWA_CMD_BAM;
  if (!strcmp(ssc,"FST")) cmd = LWA_CMD_FST;
  if (!strcmp(ssc,"CLK")) cmd = LWA_CMD_CLK;
  if (!strcmp(ssc,"REC")) cmd = LWA_CMD_REC;
  if (!strcmp(ssc,"DEL")) cmd = LWA_CMD_DEL;
  if (!strcmp(ssc,"STP")) cmd = LWA_CMD_STP;
  if (!strcmp(ssc,"GET")) cmd = LWA_CMD_GET;
  if (!strcmp(ssc,"CPY")) cmd = LWA_CMD_CPY;
  if (!strcmp(ssc,"DMP")) cmd = LWA_CMD_DMP;
  if (!strcmp(ssc,"FMT")) cmd = LWA_CMD_FMT;
  if (!strcmp(ssc,"DWN")) cmd = LWA_CMD_DWN;
  if (!strcmp(ssc,"UP_")) cmd = LWA_CMD_UP_;
  if (!strcmp(ssc,"SEL")) cmd = LWA_CMD_SEL;
  if (!strcmp(ssc,"SYN")) cmd = LWA_CMD_SYN;
  if (!strcmp(ssc,"TST")) cmd = LWA_CMD_TST;
  if (!strcmp(ssc,"BUF")) cmd = LWA_CMD_BUF;
  return cmd;
  } /* LWA_getcmd() */

/* This has to match up with the definition for MCS/Scheduler */
#define LWA_CMD_STRUCT_DATA_FIELD_LENGTH 256
struct LWA_cmd_struct {
  long int sid;      /* subsystem ID.  Must be "long int" to accomodate message queue use */
  long int ref;      /* REFERENCE number */
  int  cid;          /* command ID */
  //int  subslot;      /* subslot for  */
  int  bScheduled;   /* = 0 means "not scheduled (do as time permits)"; */
                     /* = 1 means "do as close as posible to time indicated by tv field" */  
  struct timeval tv; /* Indicates time this command is to take effect */
                     /* .tv_sec is seconds into current epoch */
                     /* .tv_usec is fractional remainder in microseconds */
  int  bAccept;      /* response: see LWA_MSELOG_TP_* */
  int  eSummary;     /* summary; see LWA_SIDSUM_* macrodefines */
  int  eMIBerror;    /* >0 if ms_mcic had a problem with the MIB.  see LWA_MIBERR_* */
  char data[LWA_CMD_STRUCT_DATA_FIELD_LENGTH]; /* DATA on way out, R-COMMENT on way back */   
  int  datalen;      /* -1 for (printable) string; 0 for zero-length; otherwise number of significant bytes */
  };


void LWA_raw2hex( char *raw, char *hex, int n ) {
  /* creates a string hex[ 0 .. 2*n ] (including \0) which is the printable */
  /* hex represenation of the bytes raw[ 0 .. n-1 ] */

  int i;
  for (i=0; i<n; i++) {
    sprintf( &(hex[2*i]), "%02x", (unsigned char) raw[i] );  
    }
  hex[2*n] = '\0';   

  return;
  } /* LWA_raw2hex() */

void LWA_timeval( 
                 struct timeval *tv, /* time as a timeval struct */
                 long int *mjd,      /* MJD */
                 long int *mpm       /* MPM */  
                 ) {
  /* converts a timeval to MJD and MPM */
  struct tm *tm;      /* from sys/time.h */
  long int a,y,m,p,q;
  
  tm = gmtime(&(tv->tv_sec));
  //printf("LWA_timeval: %02d:%02d:%02d %ld\n", tm->tm_hour, tm->tm_min, tm->tm_sec,           
  //                                            tv->tv_usec);

  /* construct the MJD field */
  /* adapted from http://paste.lisp.org/display/73536 */
  /* can check result using http://www.csgnetwork.com/julianmodifdateconv.html */
  a = (14 - tm->tm_mon) / 12; 
  y = ( tm->tm_year + 1900) + 4800 - a; // tm->tm_year is the number of years since 1900
  m = ( tm->tm_mon + 1) + (12 * a) - 3; // tm->tm_mon is number of months since Jan (0..11)
  p = tm->tm_mday + (((153 * m) + 2) / 5) + (365 * y); 
  q = (y/4) - (y/100) + (y/400) - 32045; 
  *mjd = (p+q) - 2400000.5;

  *mpm = ( (tm->tm_hour)*3600 + (tm->tm_min)*60 + (tm->tm_sec) ) * 1000
                    + (tv->tv_usec)/1000 ; 

  return;
  } /* LWA_timeval() */


void LWA_time2tv( 
               struct timeval *tv,
               long int mjd, /* MJD */
               long int mpm  /* MPM */  
               ) {
  /* given MJD and MPM, returns the associated timeval */
  /* essentially, this is LWA_timeval() in reverse */

  struct timeval tv_ref;  /* from sys/time.h */
  long int mpm_ms;
  long int mpm_s;
  long int mjd_s;

  //printf("mjd = %ld, mpm = %ld\n",mjd,mpm);

  // See http://en.wikipedia.org/wiki/Julian_day for a good explanation of the relationship between
  // MJD and Unix system time.  In a nutshell:
  // MJD [days] =  JD - 2400000.5,  where JD is the Julian Day
  // UST [days] = (JD - 2440587.5), where UST is "Unix system time"; thus:

  //LWA_time(&mjd,&mpm);

  mpm_ms = mpm % 1000;          // number of milliseconds remainder after removing integer seconds from mpm
  mpm_s  = (mpm - mpm_ms)/1000; // number of integer seconds in mpm
  mjd_s  = mjd*86400 + mpm_s;   // MJD expressed as integer seconds
  tv->tv_sec  = mjd_s - 3506716800; // integer seconds into Unix epoch
  tv->tv_usec = mpm_ms*1000;        // microseconds remainder 

  //printf("%ld %ld\n",tv->tv_sec,tv->tv_usec);
  //gettimeofday( tv, NULL );
  //printf("%ld %ld del=%ld [us]\n",tv->tv_sec,tv->tv_usec,(tv->tv_usec)-(mpm_ms*1000));

  return;
  } /* LWA_time2tv() */


void LWA_time( 
               long int *mjd,      /* MJD */
               long int *mpm       /* MPM */  
               ) {
  /* gets current time; returns MJD and MPM */

  struct timeval tv;  /* from sys/time.h */
  struct timezone tz; /* set but useless; see notes below */

  gettimeofday( &tv, &tz );     /* tz is set, but useless; see notes below */
  LWA_timeval( &tv, mjd, mpm ); /* determine MPM, MJD */
  //printf("%ld %ld\n",mjd,mpm);
  
  return;
  } /* LWA_time() */


void LWA_timeadd( struct timeval *t, long int milliseconds ) {
  long int seconds;
  seconds = milliseconds/1000;     /* break milliseconds up into integer seconds and remaining milliseconds */
  milliseconds -= (seconds*1000);
  t->tv_sec += seconds;            /* do the add and carry */
  t->tv_usec += milliseconds*1000;
  while ( (t->tv_usec) >= 1000000 ) { (t->tv_usec)-=1000000; (t->tv_sec)+=1; } 
  }

long int LWA_timediff( struct timeval t1, struct timeval t0 ) {
  long int t1ms, t0ms;

  /* remove integer number of seconds that t1 and t0 have in common */
  if ( t1.tv_sec >= t0.tv_sec ) {
      t1.tv_sec -= t0.tv_sec; t0.tv_sec = 0;
    } else {
      t0.tv_sec -= t1.tv_sec; t1.tv_sec = 0;
    }
  
  /* convert into absolute milliseconds */
  t1ms = t1.tv_sec * 1000 + t1.tv_usec/1000;   
  t0ms = t0.tv_sec * 1000 + t0.tv_usec/1000; 

  return (t1ms-t0ms);
  }

/* defined observing modes (MCS0030) */
#define LWA_OM_TRK_RADEC 1
#define LWA_OM_TRK_SOL   2
#define LWA_OM_TRK_JOV   3
#define LWA_OM_STEPPED   4
#define LWA_OM_TBW       5
#define LWA_OM_TBN       6

int LWA_getmode( char *ssc ) {
  /* ssc is a string from which to determine mode */
  /* returns the LWA observing mode ("LWA_OM_"), or 0 if there is an error */
  int mode = 0;
  if (!strncmp(ssc,"TRK_RADEC",9)) mode = LWA_OM_TRK_RADEC;
  if (!strncmp(ssc,"TRK_SOL"  ,7)) mode = LWA_OM_TRK_SOL;
  if (!strncmp(ssc,"TRK_JOV"  ,7)) mode = LWA_OM_TRK_JOV;
  if (!strncmp(ssc,"STEPPED"  ,7)) mode = LWA_OM_STEPPED;
  if (!strncmp(ssc,"TBW"      ,3)) mode = LWA_OM_TBW;
  if (!strncmp(ssc,"TBN"      ,3)) mode = LWA_OM_TBN;
  return mode;
  } /* LWA_getmode() */

void LWA_saymode( int mode, char *ssc ) {
  /* mode is one of LWA_OM_* */
  /* ssc is the associated text description */
  strcpy(ssc,"");
  if (mode==LWA_OM_TRK_RADEC) strcpy(ssc,"TRK_RADEC");
  if (mode==LWA_OM_TRK_SOL  ) strcpy(ssc,"TRK_SOL");
  if (mode==LWA_OM_TRK_JOV  ) strcpy(ssc,"TRK_JOV");
  if (mode==LWA_OM_STEPPED  ) strcpy(ssc,"STEPPED");
  if (mode==LWA_OM_TBW      ) strcpy(ssc,"TBW");
  if (mode==LWA_OM_TBN      ) strcpy(ssc,"TBN");
  return;
  } /* LWA_saymode() */

/* defined beam types (MCS0030) */
#define LWA_BT_SIMPLE            1
#define LWA_BT_MAX_SNR           2
#define LWA_BT_SPEC_DELAYS_GAINS 3

int LWA_getbeamtype( char *ssc ) {
  /* ssc is a string from which to determine beamtype */
  /* returns the LWA beam type ("LWA_BT_"), or 0 if there is an error */
  int mode = 0;
  if (!strncmp(ssc,"SIMPLE", 6)) mode = LWA_BT_SIMPLE;
  if (!strncmp(ssc,"MAX_SNR",7)) mode = LWA_BT_MAX_SNR;
  if (!strncmp(ssc,"SPEC_DELAYS_GAINS",16)) mode = LWA_BT_SPEC_DELAYS_GAINS;
  return mode;
  } /* LWA_getbeamtype() */

void LWA_saybeamtype( int mode, char *ssc ) {
  /* mode is one of LWA_BT_* */
  /* ssc is the associated text description */
  strcpy(ssc,"");
  if (mode==LWA_BT_SIMPLE ) strcpy(ssc,"SIMPLE");
  if (mode==LWA_BT_MAX_SNR) strcpy(ssc,"MAX_SNR");
  if (mode==LWA_BT_SPEC_DELAYS_GAINS) strcpy(ssc,"SPEC_DELAYS_GAINS");
  return;
  } /* LWA_saybeamtype() */

int LWA_dpoavail( 
  int dpo,            /* DP output, numbered 1 through 5 (5 is TBW/TBN) */
  struct timeval t0,  /* start time */
  long int d,         /* duration (ms) */
  char *path,         /* path to mess.dat, no trailing slash */
  char *msg )         /* text message summarizing result */
  /* reads mess.dat, determines if requested DP output is available */
  /* returns <0 if error, 0 if not available, >0 corresponds to Status of that DP output */ 
  /* -1: invalid dpo (not in range 1..5) */ 
  /* -2: mess.dat not found */ 
  {
  FILE *fp;
  char filename[256];
  char line[256];
  int dpos[5]; 
  int i;
  long int mjd,mpm;
  int dpox;
  long int durx;
  int cra;
  char project_id[256];
  char session_id[256];
  struct timeval t1,t2,t3;

  if ((dpo<1) || (dpo>5)) {
    sprintf(msg,"dpo=%d is out of range (1-5)",dpo);
    return -1;
    }

  sprintf(filename,"%s/mess.dat",path);
  if (!(fp=fopen(filename,"r"))) {
    sprintf(msg,"can't open file '%s'",filename);
    return -2;
    }

  for (i=0;i<5;i++) dpos[i]=-1; /* initialize these */

  fgets(line,256,fp); /* this should be MCS_CRA */
  fgets(line,256,fp); /* this should five numbers representing status of each beam output */
  //printf("LWA_dpoavail: <%s>\n",line);
  sscanf(line,"%d %d %d %d %d",&(dpos[0]),&(dpos[1]),&(dpos[2]),&(dpos[3]),&(dpos[4])); 

  if (dpos[dpo-1]<1) {
    fclose(fp);
    sprintf(msg,"UNAVAILABLE (status = %d)",dpos[dpo-1]);
    return 0; /* not available because this output has status <1 */
    }

  t1.tv_sec  = t0.tv_sec;
  t1.tv_usec = t0.tv_usec;
  LWA_timeadd( &t1, d );    /* t1 is now the end time for the requested session */

  fgets(line,256,fp);
  while (!feof(fp)) {
    sscanf(line,"%ld %ld %d %ld %d %s %s",&mjd,&mpm,&dpox,&durx,&cra,project_id,session_id);
    //printf("LWA_dpoavail: %ld %ld %d %ld %d <%s> <%s>\n",mjd,mpm,dpox,durx,cra,project_id,session_id);

    if (dpox==dpo) {
      LWA_time2tv( &t2, mjd, mpm ); /* get start time as a timeval */
      t3.tv_sec  = t2.tv_sec;       /* construct end time as a timeval */
      t3.tv_usec = t2.tv_usec;
      LWA_timeadd( &t3, durx );     /* end time as a timeval */     
      if ( ( LWA_timediff(t3,t0)>0 ) && ( LWA_timediff(t1,t2)>0 ) ) {
        fclose(fp);
        sprintf(msg,"conflict with project '%s' session '%s'",project_id,session_id);
        return 0;
        }
      }

    fgets(line,256,fp);
    if (strlen(line)<32) fgets(line,256,fp);

    }

  fclose(fp); 

  sprintf(msg,"available with status = %d",dpos[dpo-1]);
  return dpos[dpo-1];
  }

/*============================================*/

#endif // #ifndef MCS_H 



