
#ifndef ME_H  /* this keeps this header from getting rolled in more than once */
#define ME_H

#include "mcs.h"

/* Software module self-identification codes */
#define ME_ME_H    100 /* me.h (this file) */
#define ME_MESI    101 /* mesi() from mesi.c */
#define ME_MESIX   102 /* main() from mesix.c */
#define ME_MEOS    103 /* meos() from meos.c */
#define ME_MEOSX   104 /* main() from meosx.c */
#define ME_MEMDRE  105 /* main() from meosx.c */
#define ME_MEMDREX 106 /* main() from memdrex.c */

#include "mesi.c"
#include "memdre.c"
#include "meos.c"

/**************************************/
/*** me_bSubsystemAlive( ) ************/
/**************************************/

/* Return error codes */
#define ME_BSA_OK                 0
#define ME_BSA_INVALID_SS         1
#define ME_BSA_MEMDRE_FAIL        2
#define ME_BSA_MESI_FAIL          4
#define ME_BSA_MEMDRE_FAIL_2      8
#define ME_BSA_SUMMARY_NOT_NORMAL 16
#define ME_BSA_DIDNT_UPDATE       32

int me_bSubsystemAlive( char *ss ) {
  /* ss (input) subsystem three-letter designator */
  /* returns an error code ME_SUBSYSTEM_* indicating if subsystem is alive and responding */ 

  char val[MIB_VAL_FIELD_LENGTH];
  struct timeval t0,t1;
  long int mjd0,mjd1;
  long int mpm0,mpm1;
  int err;

  int eResult = ME_BSA_OK;
  long int reference = 0;

  /* check if 'ss' is a valid subsystem */
  /* FIXME -- ME_BSA_INVALID_SS */

  /* determine when SUMMARY field was last updated -> t0 */
  err = memdre( ss, "SUMMARY", val, &t0 );                    /* ask scheduler */ 
  if (err!=0) {
    eResult += ME_BSA_MEMDRE_FAIL; 
    return eResult;
    }
  LWA_timeval(&t0,&mjd0,&mpm0);                               /* convert t0 from timeval to MJD/MPM */
  //printf("[%d/%d] val='%s', mjd0='%ld', mpm0='%ld'\n",ME_ME_H,getpid(),val,mjd0,mpm0); 

  /* send a PNG command; this should update SUMMARY */
  err = mesi( ss, "PNG", "", "", "", &reference); 
  if (err!=0) { 
    eResult += ME_BSA_MESI_FAIL;
    return eResult;
    }
  sleep(3); /* wait for subsystem & scheduler to process */

  /* determine value of SUMMARY field now, and time of last update */
  err = memdre( ss, "SUMMARY", val, &t1 );                    /* ask scheduler */
  if (err!=0) {
    eResult += ME_BSA_MEMDRE_FAIL_2; 
    return eResult;
    }
  LWA_timeval(&t1,&mjd1,&mpm1);                               /* convert t0 from timeval to MJD/MPM */
  //printf("[%d/%d] val='%s', mjd1='%ld', mpm1='%ld'\n",ME_ME_H,getpid(),val,mjd1,mpm1);

  /* SUMMARY should be "NORMAL"... */
  if (strncmp(val,"NORMAL",6)!=0) {
    //printf("[%d/%d] SUMMARY is now '%s'\n",val),ME_ME_H,getpid(),val);
    eResult += ME_BSA_SUMMARY_NOT_NORMAL;
    }

  /* mpm1 should be different from mpm0; that is, SUMMARY should have been updated */
  if (mpm1==mpm0) {
    //printf("[%d/%d] SUMMARY did not update\n",ME_ME_H,getpid());
    eResult += ME_BSA_DIDNT_UPDATE; 
    }

  return eResult;
  } /* function bSubsystemAlive( ) */


/* me_GetMIBEntry error codes */
#define ME_GETMIBENTRY_OK          0
#define ME_GETMIBENTRY_MESI_FAIL   1
#define ME_GETMIBENTRY_MEMDRE_FAIL 2

int me_GetMIBEntry( char *ss,                        /* (input) subsystem TLA */ 
                    char *mib_label,                 /* (input) MIB label */
                    long int mib_delay_us,           /* (input) [ms] how long to wait for MIB to update */
                                                     /*         0 = do not request update; just read it */
                    char *val,                       /* (output) MIB entry */ 
                    struct timeval *t0               /* (output) Last update of MIB entry */
                  ) {

  int err;
  long int reference;
  int eResult = ME_GETMIBENTRY_OK;

  if (mib_delay_us!=0) {

    err = mesi( ss, "RPT", mib_label, "", "", &reference ); /* command update of MIB entry */ 
    if (err!=0) {
      printf("[%d/%d] FATAL: In me_GetMIBEntry, mesi('%s','RPT','%s',...) returned code %d, ref='%ld'\n",
             ME_ME_H,getpid(),ss,mib_label,err,reference);  
      eResult += ME_GETMIBENTRY_MESI_FAIL; 
      return eResult;
      }

    usleep(mib_delay_us); /* Avoid 'Failed to open dbm' problem; allow time to update MIB */
  
    }

  err = memdre( ss, mib_label, val, t0 ); /* check MIB */ 
  if (err!=0) {
    printf("[%d/%d] FATAL: In me_GetMIBEntry, memdre('%s','%s',...) returned code %d, val='%s'\n",
           ME_MEOS,getpid(),ss,mib_label,err,val);  
    eResult += ME_GETMIBENTRY_MEMDRE_FAIL; 
    return eResult;
    }

  return eResult;
  }  


#endif // #ifndef MCS_H 
